#include "TArtCalibIC.h" 
#include "TArtIC.h"
#include "TArtICPara.h"
#include "TArtBigRIPSParameters.h"
#include "TArtRawEventObject.h"
#include "TArtTOF.h" 
#include "TArtCore.h" 
#include "TArtStoreManager.h" 

#include <TROOT.h>
#include <TMath.h>
#include <TClonesArray.h>

//__________________________________________________________
TArtCalibIC::TArtCalibIC()
  :fReconstructed(false)
{
  TArtCore::Info(__FILE__,"Creating the IC detector objects...");
  sman = TArtStoreManager::Instance();
  setup = (TArtBigRIPSParameters*)sman->FindParameters("BigRIPSParameters");
  Initialize();
}

//__________________________________________________________
TArtCalibIC::~TArtCalibIC()  {
  ClearData();
}

//__________________________________________________________
void TArtCalibIC::Initialize()   {
  fICArray = new TClonesArray("TArtIC",10);
  fICArray->SetName("BigRIPSIC");
  fICArray->SetOwner();
  sman->AddDataContainer(fICArray);
}

//__________________________________________________________
void TArtCalibIC::LoadData()   {

  TArtRawEventObject *fEvent = (TArtRawEventObject *)sman->FindDataContainer("RawEvent");

  ClearData();
  for(int i=0;i<fEvent->GetNumSeg();i++){
    TArtRawSegmentObject *seg = fEvent->GetSegment(i);
    int detector = seg->GetDetector();
    if(ICE == detector) LoadData(seg);
  }

  return;

}

//__________________________________________________________
void TArtCalibIC::LoadData(TArtRawSegmentObject *seg)   {

  int device   = seg->GetDevice();
  int fpl      = seg->GetFP();
  int detector = seg->GetDetector();
  if(ICE != detector) return;

  for(int j=0;j<seg->GetNumData();j++){
    TArtRawDataObject *d = seg->GetData(j);
    int geo = d->GetGeo(); 
    int ch = d->GetCh(); 
    unsigned int val = d->GetVal();
    TArtRIDFMap mm(fpl,detector,geo,ch);
    TArtICPara *para = (TArtICPara*)setup->GetICPara(&mm);
    if(NULL == para){
      TArtCore::Info(__FILE__,"Could not find TArtICPara...: Dev:%d, %s", device, mm.GetMapInfo());
      continue;
    }
    else {
      TArtCore::Debug(__FILE__,"Find TArtICPara...: Dev:%d, %s, : %s", device, mm.GetMapInfo(), para->GetDetectorName()->Data());
    }

    int id = para->GetID();
    int nic = fICArray->GetEntries();
    TArtCore::Debug(__FILE__,"Entries in ICArray: %d, IC-ID:%d", nic, id);
    TArtIC * ic = FindIC(id);

    if(NULL==ic){
      new ((*fICArray)[nic]) TArtIC();
      ic = (TArtIC *)fICArray->At(nic);
      ic->SetID(id);
      fICParaArray.push_back(para);
    }

    // set raw data
    if(ICE == detector){
      int ic_ch = para->FindCh(&mm);
      ic->SetRawADC(ic_ch,val);
    }
  }

  fDataLoaded = true;
  return;
}

//__________________________________________________________
void TArtCalibIC::ClearData()   {
  fICArray->Clear();
  fICParaArray.clear();
  fDataLoaded = false;
  fReconstructed = false;
  return;
}

//__________________________________________________________
void TArtCalibIC::ReconstructData()   { // call after the raw data are loaded

  if(!fDataLoaded) LoadData();

  for(int i=0;i<GetNumIC();i++){

    TArtIC *ic = (TArtIC*)fICArray->At(i);
    TArtICPara *para = fICParaArray[i];

    double fAvSum = 0, fSqSum = 1;
    double fNumberOfChannelsFired = 0.0;

    ic->SetEnergySqSum(0.0);
    ic->SetEnergyAvSum(0.0);

    for(int i=0;i<NUM_IC_CHANNEL;i++) {
      unsigned int adc = ic->GetRawADC(i);
      if(adc>250) { //Set the threshold to higher values to avoid all the junk.
	fAvSum = fAvSum + adc;
	fSqSum = fSqSum * adc;
	fNumberOfChannelsFired ++;
      }
      ic->SetEnergy(i,para->GetCh2MeV(0) + para->GetCh2MeV(1)*((double)adc));
    }

    if(fNumberOfChannelsFired>0) {
      fAvSum         = fAvSum/fNumberOfChannelsFired;
      fSqSum         = TMath::Power(fSqSum,1/fNumberOfChannelsFired);
      ic->SetEnergyAvSum(para->GetCh2MeV(0) + para->GetCh2MeV(1)*fAvSum);
      ic->SetEnergySqSum(para->GetCh2MeV(0) + para->GetCh2MeV(1)*fSqSum);
    }

    // copy some information from para to data container
    ic->SetID(para->GetID());
    ic->SetDetectorName(*(para->GetDetectorName()));
    ic->SetFpl(para->GetFpl());
    ic->SetZCoef(0,para->GetZCoef(0));
    ic->SetZCoef(1,para->GetZCoef(1));
    ic->SetIonPair(para->GetIonPair());

  }

  fReconstructed = true;
  return;
}

//__________________________________________________________
TArtIC * TArtCalibIC::GetIC(int i) {
  return (TArtIC *)fICArray->At(i);
}
//__________________________________________________________
TArtICPara * TArtCalibIC::GetICPara(int i) {
  return (TArtICPara *)fICParaArray[i];
}
//__________________________________________________________
Int_t TArtCalibIC::GetNumIC() {
  return fICArray->GetEntries();
}
//__________________________________________________________
TArtIC * TArtCalibIC::FindIC(Int_t id){
  for(int i=0;i<GetNumIC();i++)
    if(id == ((TArtIC*)fICArray->At(i))->GetID())
      return (TArtIC*)fICArray->At(i);
  return NULL;
}
//__________________________________________________________
TArtIC * TArtCalibIC::FindIC(TString *n){
  for(int i=0;i<GetNumIC();i++)
    if(*n == *(((TArtIC*)fICArray->At(i))->GetDetectorName()))
      return (TArtIC*)fICArray->At(i);
  TArtCore::Info(__FILE__,"Fail to find TArtIC: %s", n->Data());
  return NULL;
}
//__________________________________________________________
TArtIC * TArtCalibIC::FindIC(char *n){
  for(int i=0;i<GetNumIC();i++)
    if(strcmp(((TArtIC*)fICArray->At(i))->GetDetectorName()->Data(),n) == 0)
      return (TArtIC*)fICArray->At(i);
  TArtCore::Info(__FILE__,"Fail to find TArtIC: %s", n);
  return NULL;
}
